<?php

namespace Zeedhi\Zmart\QueryBuilders\Mongo;

use Zeedhi\Zmart\DataSource\AggregateFunction;
use Zeedhi\Zmart\Util\ComparisonExpression;
use Zeedhi\Zmart\Util\LogicalExpression;
use Zeedhi\Zmart\Util\MathExpression;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager\Mongo\Types\Type;
use Zeedhi\Framework\ORM\DateTime;

class QueryBuilder {

    const MONGO_ID = '_id';
    const FIELD_INDICATOR = '$';

    const FUNCTION_SORT = '$sort';
    const FUNCTION_SKIP = '$skip';
    const FUNCTION_LIMIT = '$limit';
    const FUNCTION_MATCH = '$match';
    const FUNCTION_GROUP = '$group';
    const FUNCTION_PROJECT = '$project';

    const OPERATOR_COMPARISON_EQUAL = '$eq';
    const OPERATOR_COMPARISON_NOT_EQUAL = '$ne';
    const OPERATOR_COMPARISON_GREATER = '$gt';
    const OPERATOR_COMPARISON_GREATER_EQUAL = '$gte';
    const OPERATOR_COMPARISON_LOWER = '$lt';
    const OPERATOR_COMPARISON_LOWER_EQUAL = '$lte';
    const OPERATOR_COMPARISON_IN = '$in';
    const OPERATOR_COMPARISON_NOT_IN = '$nin';
    const OPERATOR_COMPARISON_BETWEEN = '$between';
    const OPERATOR_COMPARISON_LIKE_ALL = 'LIKE_ALL';

    const OPERATOR_LOGICAL_OR = '$or';
    const OPERATOR_LOGICAL_AND = '$and';

    const OPERATOR_ADD = '$add';
    const OPERATOR_SUB = '$subtract';
    const OPERATOR_MUL = '$multiply';
    const OPERATOR_DIV = '$divide';

    const AGGREGATE_SUM = '$sum';
    const AGGREGATE_AVG   = '$avg';
    const AGGREGATE_MIN   = '$min';
    const AGGREGATE_MAX   = '$max';
    const AGGREGATE_COUNT = '$count';

    const ORDER_ASC = 1;
    const ORDER_DESC = -1;

    const CONFIG_WHERE_OPERATOR = 'operator';
    const CONFIG_WHERE_COLUMN_NAME = 'columnName';
    const CONFIG_WHERE_VALUE = 'value';

    const CONFIG_MATH_EXP_LABEL = 'label';
    const CONFIG_MATH_EXP_EXPRESSION = 'mathExpression';

    const CLASS_PATH_LOGICAL_EXPRESSION = 'Zeedhi\Zmart\Util\LogicalExpression';
    const CLASS_PATH_COMPARISON_EXPRESSIION = 'Zeedhi\Zmart\Util\ComparisonExpression';

    const FORMAT_DATE_DEFAULT = 'd/m/y H:i:s';

    public static $OPERATORS_COMPARISON = array(
        ZmartFilterCriteria::EQ => self::OPERATOR_COMPARISON_EQUAL,
        ZmartFilterCriteria::NEQ => self::OPERATOR_COMPARISON_NOT_EQUAL,
        ZmartFilterCriteria::GT => self::OPERATOR_COMPARISON_GREATER,
        ZmartFilterCriteria::GTE => self::OPERATOR_COMPARISON_GREATER_EQUAL,
        ZmartFilterCriteria::LT => self::OPERATOR_COMPARISON_LOWER,
        ZmartFilterCriteria::LTE => self::OPERATOR_COMPARISON_LOWER_EQUAL,
        ZmartFilterCriteria::IN => self::OPERATOR_COMPARISON_IN,
        ZmartFilterCriteria::NOT_IN => self::OPERATOR_COMPARISON_NOT_IN,
        ZmartFilterCriteria::IS => self::OPERATOR_COMPARISON_EQUAL,

        ComparisonExpression::EQ => self::OPERATOR_COMPARISON_EQUAL,
        ComparisonExpression::NEQ => self::OPERATOR_COMPARISON_NOT_EQUAL,
        ComparisonExpression::GT => self::OPERATOR_COMPARISON_GREATER,
        ComparisonExpression::GTE => self::OPERATOR_COMPARISON_GREATER_EQUAL,
        ComparisonExpression::LT => self::OPERATOR_COMPARISON_LOWER,
        ComparisonExpression::LTE => self::OPERATOR_COMPARISON_LOWER_EQUAL
    );

    public static $SPECIAL_OPERATORS = array(
        ZmartFilterCriteria::BETWEEN,
        ZmartFilterCriteria::LIKE_ALL,
        FilterCriteria::LIKE_I
    );

    public static $AGGREGATE_FUNCTIONS = array(
        ZmartFilterCriteria::SUM => self::AGGREGATE_SUM,
        ZmartFilterCriteria::AVG => self::AGGREGATE_AVG,
        ZmartFilterCriteria::MIN => self::AGGREGATE_MIN,
        ZmartFilterCriteria::MAX => self::AGGREGATE_MAX
    );

    public static $AGGREGATE_SPECIAL_FUNCTIONS = array(
        ZmartFilterCriteria::COUNT => self::AGGREGATE_COUNT
    );

    public static $MATH_OPERATORS = array(
        MathExpression::ADD => self::OPERATOR_ADD,
        MathExpression::MUL => self::OPERATOR_MUL,
        MathExpression::SUB => self::OPERATOR_SUB,
        MathExpression::DIV => self::OPERATOR_DIV
    );

    /**
     * @param ZmartFilterCriteria $filterCriteria
     * @return array like array (
     *  array ('$group' => '_id' => 'XXXX', 'fieldName' => Array ('$sum' => '$value')),
     *  array ('$match' => array ('_id' => 0000)),
     *  array ('$sort' => array ('_id' => 1)),
     *  array ('$skip' => 100),
     *  array ('$limit' => 50)
     * )
     */
    public function buildQueryFromFilterCriteria (ZmartFilterCriteria $filterCriteria, $allowDivideByZero = false) {
        $mongoQuery = array();

        $mongoQuery = $this->buildWhere($mongoQuery, $filterCriteria);
        $mongoQuery = $this->buildGroupBy($mongoQuery, $filterCriteria);
        $mongoQuery = $this->buildMathExpressions($mongoQuery, $filterCriteria, $allowDivideByZero);
        $mongoQuery = $this->buildPostConditions($mongoQuery, $filterCriteria);
        $mongoQuery = $this->buildSort($mongoQuery, $filterCriteria);
        $mongoQuery = $this->buildSkip($mongoQuery, $filterCriteria);
        $mongoQuery = $this->buildLimit($mongoQuery, $filterCriteria);

        return $mongoQuery;
    }

    /**
     * @param array $orderBy
     * @return array|null
     */
    public function getSort(array $orderBy) {
        return !empty($orderBy) ? $this->decodeSort($orderBy) : null;
    }

    /**
     * @param array $conditions
     * @return array|null
     */
    public function getWhere(array $conditions, $columns = null) {
        if(isset($columns)){
            return $this->getWhereColumns($conditions, $columns);
        }
        else {
            return $this->getWhereNoColumns($conditions);
        }
    }

    private function getWhereColumns($conditions, $columns) {
        $finalConditions = [];
        foreach($conditions as $condition){
            if($condition["operator"] == "LIKE_ALL"){
                $conditionsExplode = explode("|",$condition["columnName"]);
                $finalConditions['$or'] = [];
                foreach($conditionsExplode as $conditionExplode){
                    $finalConditions['$or'][] = [$conditionExplode => ['$regex' => "(?i)".str_replace('%','.*', $condition["value"])]];
                    foreach($columns as $column){
                        if($column["name"] == $conditionExplode){
                            if(isset($column["isDate"]) && $column["isDate"]){
                                $date = str_replace('%','', $condition["value"]);
                                $dateValue = \DateTime::createFromFormat(self::FORMAT_DATE_DEFAULT,$date.' 00:00:00',new \DateTimeZone("UTC"));
                                if($dateValue){
                                    $date = Type::getType('mongoDate')->convertToDatabaseValue($dateValue);
                                    $finalConditions['$or'][] = [$conditionExplode => $date];
                                }
                            }else{
                                $string = str_replace('%','', $condition["value"]);
                                if(is_numeric($string)) {
                                    $finalConditions['$or'][] = [$conditionExplode => floatval($string)];
                                }
                            }
                            break;
                        }
                    }
                }
            } else {
                $key = $condition['columnName'];
                $finalConditions[$key] = $this->decodeWhereConditions([$condition])[$key];
            }
        }
        return $finalConditions;
    }

    private function getWhereNoColumns(array $conditions, $columns = null) {
        $finalConditions = [];
        foreach($conditions as $condition){
            if($condition["operator"] == "LIKE_ALL"){
                $conditionsExplode = explode("|",$condition["columnName"]);
                $finalConditions['$or'] = [];
                foreach($conditionsExplode as $conditionExplode){
                    $finalConditions['$or'][] = [$conditionExplode => ['$regex' => "(?i)".str_replace('%','.*', $condition["value"])]];
                }
            } else {
                $key = $condition['columnName'];
                $finalConditions[$key] = $this->decodeWhereConditions([$condition])[$key];
            }
        }
        return $finalConditions;
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildGroupBy(array $query, ZmartFilterCriteria $filterCriteria) {
        $groupBy = $filterCriteria->getGroupBy();
        $aggregateFunctions = $filterCriteria->getAggregateFunctions();
        if(!empty($groupBy)) {
            $query[][self::FUNCTION_GROUP] = $this->factoryGroupByClause($groupBy, $aggregateFunctions);
        }
        return $query;
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildWhere(array $query, ZmartFilterCriteria $filterCriteria) {
        $whereConditions = $filterCriteria->getConditions();
        $previousComparisonExpression = $filterCriteria->getPreviousComparisonExpression();
        if(!empty($whereConditions)) {
            $query[][self::FUNCTION_MATCH] = $this->decodeWhereConditions($whereConditions);
        }
        if($previousComparisonExpression) {
            $query[][self::FUNCTION_MATCH] = $this->decodeComparisonExpression($previousComparisonExpression);
        }

        return $query;
    }

    /**
     * @param array $groupBy
     * @param array $aggregateFunctions
     * @return array
     */
    protected function factoryGroupByClause(array $groupBy, array $aggregateFunctions) {
        $groupById = (count($groupBy) === 1)? $this->factoryFieldReference($groupBy[0]) : $this->factoryId($groupBy);
        $groupById = array(self::MONGO_ID => $groupById);
        $functions = $this->buildAggregateFunctions($aggregateFunctions);
        return array_merge($groupById, $functions);
    }

    /**
     * When grouped by more than one property mongo requires a object as Id
     * @param array $groupBy
     * @return array Mongo group by fields
     */
    protected function factoryId(array $groupBy) {
        $idObject = array();
        foreach($groupBy as $columnName) {
            $idObject[$columnName] = $this->factoryFieldReference($columnName);
        }
        return $idObject;
    }

    /**
     * Mongo field reference must be declared with the character '$' before the field name
     * @param string $columnName
     * @return string
     */
    protected function factoryFieldReference($columnName) {
        return self::FIELD_INDICATOR.$columnName;
    }

    /**
     * @param array $aggregateFunctions
     * @return array
     * @throws Exception
     */
    protected function buildAggregateFunctions(array $aggregateFunctions) {
        $functions = array();
        if(!empty($aggregateFunctions)) {
            foreach($aggregateFunctions as $aggregateConfiguration) {
                $functionLabel = $this->getAggConfigLabel($aggregateConfiguration);
                $functions[$functionLabel] = $this->factoryAggregateFunction($aggregateConfiguration);
            }
        }
        return $functions;
    }

    /**
     * @param AggregateFunction $aggregateFunction
     * @return array
     * @throws Exception
     */
    protected function factoryAggregateFunction(AggregateFunction $aggregateFunction) {
        $functionName = $this->getAggConfigFunction($aggregateFunction);

        if($this->isAggregateFunction($functionName)) {
            $functionToReturn = $this->factoryAggregateNormalFunction($functionName, $aggregateFunction);
        } else if($this->isAggregateSpecialFunction($functionName)){
            $functionToReturn = $this->factoryAggregateSpecialFunction($functionName);
        } else {
            throw Exception::invalidFunction($functionName);
        }

        return $functionToReturn;
    }

    /**
     * @param AggregateFunction $aggregateFunction
     * @return string
     */
    protected function getAggConfigFunction(AggregateFunction $aggregateFunction) {
        return $aggregateFunction->getFunction();
    }

    /**
     * @param AggregateFunction $aggregateFunction
     * @return string
     */
    protected function getAggConfigContent(AggregateFunction $aggregateFunction) {
        return $aggregateFunction->getContent();
    }

    /**
     * @param AggregateFunction $aggregateFunction
     * @return string
     */
    protected function getAggConfigLabel(AggregateFunction $aggregateFunction) {
        return $aggregateFunction->getLabel();
    }

    /**
     * @param $functionName
     * @param $aggregateConfiguration
     * @return array
     */
    protected function factoryAggregateNormalFunction($functionName, $aggregateConfiguration) {
        $content = $this->getAggConfigContent($aggregateConfiguration);
        $aggregateFunction = self::$AGGREGATE_FUNCTIONS[$functionName];
        $formattedContent = $this->factoryValueByType($content);
        return array($aggregateFunction => $formattedContent);
    }

    /**
     * @param MathExpression $mathExpression
     * @return array
     */
    protected function factoryMathExpression(MathExpression $mathExpression, $allowDivideByZero = false) {
        $firstValue = $this->getMathExpressionFirstValue($mathExpression);
        $firstValue = $this->factoryValueByType($firstValue, $allowDivideByZero);
        $secondValue = $this->getMathExpressionSecondValue($mathExpression);
        $secondValue = $this->factoryValueByType($secondValue, $allowDivideByZero);
        $pipeline = array($this->translateMathOperator($mathExpression) => array($firstValue, $secondValue));
        if($allowDivideByZero && $mathExpression->getOperator() == MathExpression::DIV){
            $expression = array();
            $expression[0] = array('$eq' => array($secondValue, 0));
            $expression[1] = 0;
            $expression[2] = $pipeline;
            $returnArray = array('$cond' => $expression);
        }else {
            $returnArray = $pipeline;
        }

        return $returnArray;
    }

    /**
     * @param $value
     * @return array|int|string
     * @throws Exception
     */
    protected function factoryValueByType($value, $allowDivideByZero = false) {
        switch(true) {
            case is_string($value) :
                $formattedContent = $this->factoryFieldReference($value);
                break;

            case is_numeric($value) :
                $formattedContent = $value;
                break;

            case $value instanceof MathExpression :
                $formattedContent = $this->factoryMathExpression($value, $allowDivideByZero);
                break;

            default :
                throw Exception::invalidContentType($this->getInstanceAsString($value));
        }
        return $formattedContent;
    }

    /**
     * @param MathExpression $mathExpression
     * @return string
     */
    protected function translateMathOperator(MathExpression $mathExpression) {
        return self::$MATH_OPERATORS[$this->getMathExpressionOperator($mathExpression)];
    }

    /**
     * @param $var
     * @return string
     */
    protected function getInstanceAsString($var) {
        return gettype($var) === 'object'? get_class($var) : gettype($var);
    }

    /**
     * @param MathExpression $mathExpression
     * @return string
     */
    protected function getMathExpressionOperator(MathExpression $mathExpression) {
        return $mathExpression->getOperator();
    }

    /**
     * @param MathExpression $mathExpression
     * @return MathExpression|int|string
     */
    protected function getMathExpressionFirstValue(MathExpression $mathExpression) {
        return $mathExpression->getFirstValue();
    }

    /**
     * @param MathExpression $mathExpression
     * @return MathExpression|int|string
     */
    protected function getMathExpressionSecondValue(MathExpression $mathExpression) {
        return $mathExpression->getSecondValue();
    }

    /**
     * @param $functionName
     * @return array
     */
    protected function factoryAggregateSpecialFunction($functionName) {
        $aggregateSpecialFunction = self::$AGGREGATE_SPECIAL_FUNCTIONS[$functionName];
        return $this->buildAggregateSpecialFunction($aggregateSpecialFunction);
    }

    /**
     * @param $functionName
     * @return bool
     */
    protected function isAggregateFunction ($functionName) {
        return isset(self::$AGGREGATE_FUNCTIONS[$functionName]);
    }

    /**
     * @param $functionName
     * @return bool
     */
    protected function isAggregateSpecialFunction($functionName) {
        return isset(self::$AGGREGATE_SPECIAL_FUNCTIONS[$functionName]);
    }

    /**
     * @param $function
     * @return array
     */
    protected function buildAggregateSpecialFunction($function) {
        $specialFunction = array();
        if($function == self::AGGREGATE_COUNT) {
            $specialFunction = array(self::AGGREGATE_SUM => 1);
        }
        return $specialFunction;
    }


    protected function buildMathExpressions(array $query, ZmartFilterCriteria $filterCriteria, $allowDivideByZero = false) {
        $mathExpressions = $filterCriteria->getMathExpressions();
        $aggFunctions = $filterCriteria->getAggregateFunctions();
        if(!empty($mathExpressions)) {
            $query[][self::FUNCTION_PROJECT] = $this->factoryMathExpressions($mathExpressions, $aggFunctions, $allowDivideByZero);
        }
        return $query;
    }

    protected function factoryMathExpressions(array $mathExpressions, array $aggFunctions, $allowDivideByZero = false) {
        $factoredMathExpressions = array();
        foreach($mathExpressions as $mathExpressionConfig) {
            $factoredMathExpressions[$this->getMathExpConfigLabel($mathExpressionConfig)] =
                $this->factoryMathExpression($this->getMathExpConfigExpression($mathExpressionConfig), $allowDivideByZero);
        }
        foreach ($aggFunctions as $aggFunction) {
            $aggFunctionLabel = $this->getAggConfigLabel($aggFunction);
            $factoredMathExpressions[$aggFunctionLabel] = $this->factoryFieldReference($aggFunctionLabel);
        }
        return $factoredMathExpressions;
    }

    /**
     * @param $mathExpressionConfig
     * @return string
     */
    protected function getMathExpConfigLabel($mathExpressionConfig) {
        return $mathExpressionConfig[self::CONFIG_MATH_EXP_LABEL];
    }

    /**
     * @param $mathExpressionConfig
     * @return MathExpression
     */
    protected function getMathExpConfigExpression($mathExpressionConfig) {
        return $mathExpressionConfig[self::CONFIG_MATH_EXP_EXPRESSION];
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildPostConditions(array $query, ZmartFilterCriteria $filterCriteria) {
        $whereConditions = $filterCriteria->getPostConditions();
        $postComparisonExpression = $filterCriteria->getPostComparisonExpression();
        if(!empty($whereConditions)) {
            $query[][self::FUNCTION_MATCH] = $this->decodeWhereConditions($whereConditions);
        }
        if($postComparisonExpression) {
            $query[][self::FUNCTION_MATCH] = $this->decodeComparisonExpression($postComparisonExpression);
        }

        return $query;
    }

    /**
     * @param array $whereConditions
     * @return array
     * @throws Exception
     */
    protected function decodeWhereConditions(array $whereConditions) {
        $decodedConditions = array();
        foreach($whereConditions as $condition) {
            $columnName = $this->getWhereConfigColumnName($condition);
            $decodedConditions[$columnName] = $this->factoryWhereCondition($condition);
        }

        return $decodedConditions;
    }

    /**
     * @param array $condition
     * @return array
     * @throws Exception
     */
    protected function factoryWhereCondition(array $condition) {
        $operatorName = $this->getWhereConfigOperator($condition);
        if($this->isOperatorMapped($operatorName)) {
            $whereCondition = array(self::$OPERATORS_COMPARISON[$operatorName] => $this->getWhereConfigValue($condition));
        } else if ($this->isSpecialOperatorMapped($operatorName)) {
            $whereCondition = $this->factorySpecialWhereCondition($condition);
        } else {
            throw Exception::invalidOperator($operatorName);
        }

        return $whereCondition;
    }

    /**
     * @param LogicalExpression|ComparisonExpression $comparisonExpression
     * @return array
     * @throws Exception
     */
    protected function decodeComparisonExpression($comparisonExpression) {
        return $this->factoryLogicalOrComparisonExpression($comparisonExpression);
    }

    /**
     * @param LogicalExpression $logicalExpression
     * @return array|ComparisonExpression|LogicalExpression
     */
    protected function factoryLogicalExpression(LogicalExpression $logicalExpression) {
        $expressionToReturn = array();

        $operator = $logicalExpression->getOperator();
        $expressions = array();

        foreach($logicalExpression->getExpressions() as $expression) {
            $expressions[] = $this->factoryLogicalOrComparisonExpression($expression);
        }

        $expressionToReturn[$this->translateLogicalOperator($operator)] = $expressions;

        return $expressionToReturn;
    }

    protected function translateLogicalOperator($operator) {
        if($operator === LogicalExpression::OP_OR) {
            $operator = self::OPERATOR_LOGICAL_OR;
        } else if($operator === LogicalExpression::OP_AND) {
            $operator = self::OPERATOR_LOGICAL_AND;
        }

        return $operator;
    }

    /**
     * @param LogicalExpression|ComparisonExpression $comparisonExpression
     * @return array
     * @throws Exception
     */
    protected function factoryLogicalOrComparisonExpression($comparisonExpression) {
        $expression = array();
        if(get_class($comparisonExpression) === self::CLASS_PATH_LOGICAL_EXPRESSION) {
            $expression = $this->factoryLogicalExpression($comparisonExpression);
        } else if(get_class($comparisonExpression) === self::CLASS_PATH_COMPARISON_EXPRESSIION){
            $expression = $this->factoryComparisonExpression($comparisonExpression);
        }
        return $expression;
    }

    /**
     * @param ComparisonExpression $comparisonExpression
     * @return array
     * @throws Exception
     */
    protected function factoryComparisonExpression(ComparisonExpression $comparisonExpression) {
        $operator = $comparisonExpression->getOperator();
        if($this->isOperatorMapped($operator)) {
            $expression = array();
            $firstValue = $comparisonExpression->getFirstValue();
            $secondValue = $comparisonExpression->getSecondValue();
            $operator = $this->translateComparisonOperator($operator);
            $expression[$firstValue] = array($operator => $secondValue);
            return $expression;
        } else {
            throw Exception::invalidOperator($operator);
        }
    }

    /**
     * @param $value
     * @return string|mixed
     */
    protected function ifStringFactoryReference($value) {
        return is_string($value) ? $this->factoryFieldReference($value) : $value;
    }

    /**
     * @param $operator
     * @return string
     */
    protected function translateComparisonOperator($operator) {
        return self::$OPERATORS_COMPARISON[$operator];
    }

    /**
     * @param $operatorName
     * @return bool
     */
    protected function isOperatorMapped($operatorName) {
        return isset(self::$OPERATORS_COMPARISON[$operatorName]);
    }

    /**
     * @param $operatorName
     * @return bool
     */
    protected function isSpecialOperatorMapped($operatorName) {
        return in_array($operatorName, self::$SPECIAL_OPERATORS);
    }

    /**
     * @param array $condition
     * @return string
     */
    protected function getWhereConfigColumnName(array $condition) {
        return $condition[self::CONFIG_WHERE_COLUMN_NAME];
    }

    /**
     * @param array $condition
     * @return string
     */
    protected function getWhereConfigOperator(array $condition) {
        return $condition[self::CONFIG_WHERE_OPERATOR];
    }

    /**
     * @param array $condition
     * @return string
     */
    protected function getWhereConfigValue(array $condition) {
        return $condition[self::CONFIG_WHERE_VALUE];
    }

    /**
     * @param array $condition
     * @return array
     * @throws Exception
     */
    protected function factorySpecialWhereCondition(array $condition) {
        $specialOperator = $this->getWhereConfigOperator($condition);
        $specialWhere = array();

        if($specialOperator === ZmartFilterCriteria::BETWEEN) {
            $specialWhere = $this->getBetweenCondition($condition);
        } else {
            $specialWhere = $this->getLikeAllCondition($condition['value']);
        }

        return $specialWhere;
    }

    /**
     * @param $condition
     * @return array
     */
    protected function getBetweenCondition($condition) {
        $betweenValue = $this->getWhereConfigValue($condition);
        if(!$this->isValidBetweenValue($betweenValue)) {
            throw Exception::invalidBetween($betweenValue);
        }

        $conditionValue = array(
            self::$OPERATORS_COMPARISON[ZmartFilterCriteria::GTE] => $betweenValue[0],
            self::$OPERATORS_COMPARISON[ZmartFilterCriteria::LTE] => $betweenValue[1]
        );
        return $conditionValue;
    }

    /**
     * @param $condition
     * @return array
     */
    protected function getLikeAllCondition($conditionValue) {
        $conditionValue = str_replace('%', '', $conditionValue);
        return array(
            '$regex' => ".*{$conditionValue}.*"
        );
    }

    /**
     * @param $betweenValue
     * @return bool
     */
    protected function isValidBetweenValue($betweenValue) {
        return (is_array($betweenValue) && isset($betweenValue[0]) && isset($betweenValue[1]));
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildSort(array $query, ZmartFilterCriteria $filterCriteria) {
        $sort = $filterCriteria->getOrderBy();
        $groupBy = $filterCriteria->getGroupBy();
        if (!empty($sort)) {
            $query[][self::FUNCTION_SORT] = $this->decodeSort($sort, $groupBy);
        }
        return $query;
    }

    /**
     * @param array $sort
     * @param array $groupBy
     * @return array
     * @throws Exception
     */
    protected function decodeSort(array $sort, array $groupBy = array()) {
        $sortArray = array();
        foreach($sort as $columnName => $direction) {
            if(in_array($columnName, $groupBy)) {
                if(count($groupBy) === 1) {
                    $sortArray[self::MONGO_ID] = $this->decodeDirection($direction);
                }
                else {
                    $sortArray[self::MONGO_ID.".".$columnName] = $this->decodeDirection($direction);
                }
            } else {
                $sortArray[$columnName] = $this->decodeDirection($direction);
            }
        }
        return $sortArray;
    }

    /**
     * @param $direction
     * @return int
     * @throws Exception
     */
    protected function decodeDirection($direction) {
        if($this->isAscendantDirection($direction)) {
            $sortField =  self::ORDER_ASC;
        } else if ($this->isDescendantDirection($direction)) {
            $sortField =  self::ORDER_DESC;
        } else {
            throw Exception::invalidSortDirection($direction);
        }
        return $sortField;
    }

    /**
     * @param $direction
     * @return bool
     */
    protected function isAscendantDirection($direction) {
        return ($direction == ZmartFilterCriteria::ORDER_ASC || $direction == self::ORDER_ASC);
    }

    /**
     * @param $direction
     * @return bool
     */
    protected function isDescendantDirection($direction) {
        return ($direction == ZmartFilterCriteria::ORDER_DESC || $direction == self::ORDER_DESC);
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildSkip (array $query, ZmartFilterCriteria $filterCriteria) {
        $limit = $filterCriteria->getPageSize();
        $page = $filterCriteria->getPage();
        if($page && $limit > 0 && $page > 1) {
            $query[][self::FUNCTION_SKIP] = $limit*($page-1);
        }
        return $query;
    }

    /**
     * @param array $query
     * @param ZmartFilterCriteria $filterCriteria
     * @return array
     */
    protected function buildLimit(array $query, ZmartFilterCriteria $filterCriteria) {
        $limit = $filterCriteria->getPageSize();
        if($limit > 0) {
            $query[][self::FUNCTION_LIMIT] = $limit;
        }
        return $query;
    }
}